<?php

defined('_ENGINE') or die('Access Denied');

/**
 * Prints synchronization data to be readed by BoWoB server.
 * @return void.
 */
function bowob_server_sync()
{
  header('Cache-Control: no-cache, must-revalidate');
  header('Content-Type: text/plain; charset=utf-8');

  include(dirname(realpath(__FILE__)) . SE_DS . 'bowob_api.php');

  $request = Zend_Controller_Front::getInstance()->getRequest();

  echo bowob_api_get_sync_data(
    $request->getParam('id', '-1'),
    $request->getParam('sync', ''),
    $request->getParam('name', '') == '1',
    $request->getParam('avatar', '') == '1',
    $request->getParam('friends', '') == '1'
  );
}

/**
 * Creates a synchronization record and prints record identifiers to be readed by BoWoB client.
 * @return void.
 */
function bowob_client_sync()
{
  header('Cache-Control: no-cache, must-revalidate');
  header('Content-Type: text/plain');

  include(dirname(realpath(__FILE__)) . SE_DS . 'bowob_api.php');

  $request = Zend_Controller_Front::getInstance()->getRequest();

  echo bowob_api_new_sync(
    $request->getParam('nick', ''),
    $request->getParam('name', '') == '1',
    $request->getParam('avatar', '') == '1'
  );
}

/**
 * Redirects to login page.
 * @return void.
 */
function bowob_redirect_login()
{
  include_once( APPLICATION_PATH . '/application/libraries/Zend/Controller/Action/Helper/Redirector.php');

  $redirector = new Zend_Controller_Action_Helper_Redirector();
 
  $redirector->gotoUrl('login');

  exit();
}

/**
 * Redirects to user profile page.
 * @return void.
 */
function bowob_redirect_profile()
{
  include_once( APPLICATION_PATH . '/application/libraries/Zend/Controller/Action/Helper/Redirector.php');

  $request = Zend_Controller_Front::getInstance()->getRequest();
  $redirector = new Zend_Controller_Action_Helper_Redirector();

  $userId = $request->getParam('id', '-1');
  $profileLink = '';

  if($userId > 0)
    $profileLink = '/profile/'.$userId;

  $redirector->gotoUrl($profileLink);

  exit();
}

/**
 * Gets BoWoB HTML code for show the chat.
 * @return string The HTML code.
 */
function bowob_code($app_id, $server_address)
{
  include(dirname(realpath(__FILE__)) . SE_DS . 'bowob_api.php');

  return bowob_api_get_code($app_id, $server_address);
}

/**
 * Checks if current user is logued.
 * @return boolean User is logued.
 */
function bowob_is_user_logued()
{
  return (boolean)Engine_Api::_()->user()->getViewer()->getIdentity();
}

/**
 * Gets current user id.
 * @return int User id.
 */
function bowob_get_user_id()
{
  return Engine_Api::_()->user()->getViewer()->getIdentity();
}

/**
 * Gets current user nick.
 * @return string User nick.
 */
function bowob_get_user_nick()
{
  if (bowob_is_user_logued())
    return Engine_Api::_()->user()->getUser(bowob_get_user_id())->username;
  else
    return '';
}

/**
 * Gets current user name.
 * @return string User name.
 */
function bowob_get_user_name()
{
  if (bowob_is_user_logued())
    return Engine_Api::_()->user()->getUser(bowob_get_user_id())->getTitle();
  else
    return '';
}

/**
 * Gets current user avatar url.
 * @return string User avatar.
 */
function bowob_get_user_avatar()
{
  $baseUrl = Zend_Controller_Front::getInstance()->getRequest()->getBasePath();

  if (bowob_is_user_logued())
  {
    $userImage = Engine_Api::_()->user()->getViewer()->getPhotoUrl();
    if ($userImage)
      return $userImage;
  }

  $noImage = $baseUrl . '/application/modules/User/externals/images/nophoto_user_thumb_profile.png';
  if (file_exists($noImage))
    return $noImage;
  else
    return '';
}

/**
 * Gets current user friends.
 * @param int $id User id.
 * @param string $separator Separator between nicks.
 * @return string User friends.
 */
function bowob_get_user_friends($id, $separator)
{
  if($id <= 0)
  {
    return '';
  }

  $output = '';

  $friendsTable = Engine_Api::_()->getDbtable('membership', 'user');
  $friendsName = $friendsTable->info('name');

  $select = new Zend_Db_Select($friendsTable->getAdapter());
  $select
    ->from($friendsName, 'user_id')
    ->join($friendsName, "`{$friendsName}`.`user_id`=`{$friendsName}_2`.user_id", null)
    ->where("`{$friendsName}`.resource_id = ?", $id)
    ->where("`{$friendsName}_2`.resource_id = ?", $id)
  ;

  foreach( $select->query()->fetchAll() as $data ) {
    $output .= Engine_Api::_()->user()->getUser($data['user_id'])->username . $separator;
  }

  return $output;
}

/**
 * Stores a synchronization record in database.
 * @param string $auth Record auth.
 * @param int $creation Record creation time.
 * @param int $user_id Record user id.
 * @param string $user_nick Record user nick.
 * @param string $user_name Record user name.
 * @param string $user_avatar Record user avatar.
 * @param int $user_type Record user type.
 * @return int Record id.
 */
function bowob_store_sync($auth, $creation, $user_id, $user_nick, $user_name, $user_avatar, $user_type)
{
  return Engine_Api::_()->getDbtable('bowobs', 'bowob')->insert(array(
             'auth' => $auth,
             'creation' => (int)$creation,
             'user_id' => (int)$user_id,
             'user_nick' => $user_nick,
             'user_name' => $user_name,
             'user_avatar' => $user_avatar,
             'user_type' => $user_type,
           )
         );
}

/**
 * Extracts a synchronization record from database.
 * @param int $id Record id.
 * @param string $auth Record auth.
 * @param int $expiration Record expiration time.
 * @return array Record values.
 */
function bowob_extract_sync($id, $auth, $expiration)
{

  Engine_Api::_()->getDbtable('bowobs', 'bowob')->delete(array(
      'creation < ?' => (int)$expiration,
    )
  );
    
  $bowobTable = Engine_Api::_()->getDbtable('bowobs', 'bowob');
  $bowobTableName = $bowobTable->info('name');
  $select = new Zend_Db_Select($bowobTable->getAdapter());
  $select
    ->from($bowobTableName, '*')
    ->where("`{$bowobTableName}`.id = ?", (int)$id)
  ;

  $rs = $select->query()->fetchAll();
  $rs = $rs[0];
 
  if(!$rs || $rs['auth'] != $auth)
  {
    return array();
  }
  else
  {
    Engine_Api::_()->getDbtable('bowobs', 'bowob')->delete(array(
        'id < ?' => (int)$id,
      )
    );

    return array(
      'user_id' => $rs['user_id'],
      'user_nick' => $rs['user_nick'],
      'user_name' => $rs['user_name'],
      'user_avatar' => $rs['user_avatar'],
      'user_type' => $rs['user_type'],
    );
  }
}
